package main

import (
	"bufio"
	"fmt"
	"io"
	"net"
	"os"
	"os/exec"
	"os/signal"
	"strings"
	"syscall"
	"log"

	"github.com/creack/pty"
)

func main() {
	log.Println("\033[32mStarting Server")
	sigs := make(chan os.Signal, 1)
	signal.Notify(sigs, syscall.SIGTERM)

	var minram, maxram, args, stop, jar string
	for _, e := range os.Environ() {
		pair := strings.SplitN(e, "=", 2)
		switch pair[0] {
		case "MIN_MEM":
			minram = "-Xms" + pair[1]
		case "MAX_MEM":
			maxram = "-Xmx" + pair[1]
		case "ARGS":
			args = pair[1]
		case "STOP":
			stop = pair[1]
		case "JAR":
			jar = pair[1]
		}

	}
	log.Printf("Ram: Min: %s Max: %s Stop: %s Jar: %s\n\r", minram, maxram, stop, jar)

	

	if _, err := os.Stat("/server/"+jar); os.IsNotExist(err) {
		log.Println("\033[31mJar \"" + jar + "\" not found")
		os.Exit(1)
	}
	
	var cmd *exec.Cmd
	if args == "" {
		cmd = exec.Command("java", minram, maxram, "-jar", jar)
	} else {
		cmd = exec.Command("java", minram, maxram, args, "-jar", jar)
		log.Printf("Args: %s \n\r", args)
	}

	cmd.Dir = "/server"
	cmd.Stdout = os.Stdout
	tty, err := pty.Start(cmd)
	ln, _ := net.Listen("tcp", ":8081")

	if err != nil {
		panic(err)
	
	}

	defer func() {
		cmd.Process.Kill()
		cmd.Process.Wait()
		tty.Close()
		ln.Close()
	}()

	go func() {
		sig := <-sigs
		if sig == syscall.SIGTERM {
			fmt.Println("\033[31mStopping server")
			tty.WriteString(stop + "\n\r")
		}
	}()

	go func() {
		//accept connections from clients
		for {
			conn, err := ln.Accept()
			if err != nil {
				continue
			}
			go handleClient(conn, tty)
		}
	}()

	go func() {
		io.Copy(tty, os.Stdin)
	}()

	cmd.Wait()
	tty.Write([]byte("\033[31mServer Stopped"))
	os.Exit(0)
}

func handleClient(conn net.Conn, tty *os.File) {
	defer conn.Close()
	cmd, _ := bufio.NewReader(conn).ReadString('\n')
	tty.Write([]byte(cmd))
}
